#
# spec file for package uki
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# needsbinariesforbuild
# needssslcertforbuild


%global debug_package %{nil}

# Flavors conditions
# -------------------------
%define flavor default%{nil}
%if "%{flavor}" != ""
%define name_suffix -%{flavor}
%endif
# -------------------------
# Kernel name
# -------------------------
%ifarch %ix86 x86_64
%define ker_name vmlinuz
%endif
%ifarch ppc ppc64 ppc64le
%define ker_name vmlinux
%endif
%ifarch s390 s390x
%define ker_name image
%endif
%ifarch %arm
%define ker_name zImage
%endif
%ifarch aarch64 riscv64
%define ker_name Image
%endif
# -------------------------
# usefull variables
# -------------------------
%global uname %(ls /lib/modules | head -n 1)
%global kernel_version %(rpm -q --queryformat '%%{VERSION}' kernel-%{flavor})
%global initrd_pkg static-initrd-generic-%{flavor}-unsigned
%global initrd_version %(rpm -q --queryformat '%%{VERSION}' %{initrd_pkg} | sed 's|_k.*||')
%global initrd_release %(rpm -q --queryformat '%%{RELEASE}' %{initrd_pkg} | cut -d "." -f1)
%define etf /boot/efi
%define distro opensuse
%define efi_dir EFI/%{distro}
%define efi_path %{etf}/%{efi_dir}
%define efi_name uki-%{flavor}-%{initrd_version}-%{initrd_release}
%define url https://uapi-group.org/specifications/specs/unified_kernel_image
%define uki_install_dir %{kernel_module_directory}/%{uname}
%global cert_install %(if [ -f %{_sourcedir}/_projectcert.crt ]; then echo 1; else echo 0; fi)
%define cert_name uki-%{flavor}
%define cert_install_dir %{_datarootdir}/unified/certs
# -------------------------
# Module Kernel macro
# -------------------------
# TW is usrmerged
%if %{undefined usrmerged} && 0%{?suse_version} >= 1550
%define usrmerged 1
%endif

%if 0%{?usrmerged}
%define kernel_module_directory /usr/lib/modules
%else
%define kernel_module_directory /lib/modules
%endif

# source_rel is the package release string, without the rebuild counter
# generated by the build service. If the release string has a non-digit
# suffix, we keep that suffix and strip the rightmost digit component.
# This is used in KOTD builds: 2.1.g1234567 -> 2.g1234567
%define source_rel %(echo %release | sed -r 's/\\.[0-9]+($|\\.[^.]*[^.0-9][^.]*$)/\\1/')
# -------------------------

Name:           uki%{?name_suffix}
Version:        %{initrd_version}_%{initrd_release}_k%{kernel_version}
Release:        3.7
Summary:        Signed Unified Kernel Image
License:        Apache-2.0
URL:            %{url}
Group:          System/Kernel
Provides:       uki:%{cert_install_dir}
%if "%{flavor}" == ""
ExclusiveArch:  do_not_build
%else
ExclusiveArch:  x86_64
%endif
BuildRequires:  pesign-obs-integration
# -------------------------
# Needed to have the ukify tool
# -------------------------
BuildRequires:  systemd, systemd-experimental
BuildRequires:  %{python_module cryptography}
BuildRequires:  %{python_module pefile}
# -------------------------
BuildRequires:  %{initrd_pkg}
BuildRequires:  gptfdisk
BuildRequires:  kernel-%{flavor}
BuildRequires:  openssl
BuildRequires:  pesign
BuildRequires:  systemd-boot
BuildRequires:  tpm2.0-tools
BuildRequires:  udev
BuildRequires:  uki-tool >= 1.5.0
BuildRequires:  update-bootloader
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
Provides:       %{name} = %{version}-%{source_rel}
Provides:       multiversion(kernel)

Requires:       uki-tool >= 1.5.0
# Needed in post to change the type of the partition
Requires:       gptfdisk
Requires:       shim >= 11
# Needed by systemd-pcrphase at boot
Requires:       pkgconfig(tss2-tcti-device)
# Install the UKI with the correct kernel version
Requires:       kernel-%{flavor} = %{kernel_version}

%description
Generic signed Unified Kernel Image based on kernel %{flavor}

%prep
# Get the certificates used to sign
if test -e %{_sourcedir}/_projectcert.crt ; then
    cp %{_sourcedir}/_projectcert.crt ./%{cert_name}.crt
    openssl x509 -in ./%{cert_name}.crt -outform DER -out ./%{cert_name}.der
fi
# Get the static initrd from RPM.
rpm2cpio /.build.binaries/%{initrd_pkg}.rpm | cpio -idmv

%build
staticinitrd="./usr/share/initrd/$(basename ./usr/share/initrd/static-initrd-generic-*.unsigned)"
if [ ! -f "$staticinitrd" ]; then
    echo "No static initrd extracted"
    exit 1
fi
uki-tool create \
    -k %{uname} \
    -i "$staticinitrd" \
    -c "rw quiet systemd.show_status=1 console=ttyS0,115200 console=tty0"\
    -n %{efi_name}.efi \
    -o .

%install
export BRP_PESIGN_FILES="%{uki_install_dir}/%{efi_name}.efi"
install -d -m 0700 %{buildroot}%{uki_install_dir}/
install -m 0755 ./%{efi_name}.efi \
    %{buildroot}%{uki_install_dir}/%{efi_name}.efi
install -d -m 0700 %{buildroot}%{uki_install_dir}/%{efi_name}.efi.extra.d

%if %{cert_install}
install -d -m 0700 %{buildroot}%{cert_install_dir}
fpr=$(openssl x509 -sha1 -fingerprint -inform DER -noout -in ./%{cert_name}.der \
    | cut -c 18- | cut -d ":" -f 1,2,3,4 | sed 's/://g')
install -m 644 ./%{cert_name}.der \
    %{buildroot}%{cert_install_dir}/%{efi_name}-${fpr}.crt
%endif

%posttrans
# Change type of the file system partition from 8300 to 8304
dev_name=$(df -h / | tail -1 | cut -d ' ' -f1)
dev_part=${dev_name::-1}
dev_no=${dev_name: -1}
if [[ $dev_no && $dev_part ]] &&\
   [[ -b "${dev_part}${dev_no}" ]]; then
    echo "Change partition type of ${dev_part}${dev_no} in 8304"
    sgdisk -t $dev_no:8304 $dev_part || true
fi
# Add menuentry to bootloaders
entry_added=0
if [ $(command -v sdbootutil) ] && [ "$(sdbootutil bootloader)" == "systemd-boot" ]; then
  if (command -v sdbootutil > /dev/null 2>&1); then
    if uki-tool -q sdboot \
      --add \
      --all-ukis \
      --efi %{efi_dir}; then
        echo "Entry for UKI has been added to sdboot"
        entry_added=1
    else
        echo "Entry for UKI has not been added to sdboot"
        exit 0
    fi
  fi
else
  if (command -v grub2-mkconfig > /dev/null 2>&1); then
    if uki-tool -q grub2 \
      --add \
      --all-ukis \
      --efi %{efi_dir}; then
        echo "Entry for UKI has been added to Grub"
        entry_added=1
    else
        echo "Entry for UKI hasn't been added to Grub"
        exit 0
    fi
  fi
fi
cert_file=$(basename %{cert_install_dir}/%{efi_name}-*.crt)
cert_file_p="%{cert_install_dir}/${cert_file}"
if test "$entry_added" = "1" -a -f ${cert_file_p} ; then
    echo -e "\033[0;32mTo enroll the uki certificate key please run:\033[0m"
    echo -e "\033[0;32mmokutil \
--import ${cert_file_p}\033[0m"
fi

%preun
if [ $(command -v sdbootutil) ] && [ "$(sdbootutil bootloader)" == "systemd-boot" ]; then
  if uki-tool -q sdboot \
    --remove \
    --all-ukis \
    --efi %{efi_dir}; then
    echo "Entry for UKI has been removed from sdboot"
  else
    echo "Entry for UKI has not been removed from sdboot"
    exit 0
  fi
else
  if (command -v grub2-mkconfig > /dev/null 2>&1); then
    if uki-tool -q grub2 \
      --remove \
      --all-ukis \
      --efi %{efi_dir}; then
        echo "Entries for UKI has been removed from grub2"
    else
        echo "Entries for UKI has not been removed from grub2"
        exit 0
    fi
  fi
fi

%files
%defattr(-,root,root)
%attr(0644, root, root) %{kernel_module_directory}/%{uname}/%{efi_name}.efi
%{kernel_module_directory}/%{uname}/%{efi_name}.efi.extra.d
%if %{cert_install}
%dir %{_datarootdir}/unified
%dir %{_datarootdir}/unified/certs
%{cert_install_dir}/%{efi_name}-*.crt
%endif

%changelog
* Sat May 24 2025 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Remove global uki extra dir
  * Global extra dir will be manage in another package.
* Fri May 23 2025 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Add vanilla flavor
  - BuildRequires to the new initrd package.
* Wed May 14 2025 Valentin Lefebvre <valentin.lefebvre@suse.com>
- multi-versioning
  - Add source_rel and use it for adding Provides with good versioning.
  - Add uname in files
  - Provides multiversion(kernel)
* Wed Jan  8 2025 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Fix posttrans script condition
* Tue Dec 31 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Do not sign uki for local build. Remove local certificate.
* Thu Dec 19 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- scriptlets: install and remove all ukis
  * Need to use version 1.4.1 of uki-tool
- Get the kernel version from the package installed
- name_suffix defined only if flavor is defined
* Mon Sep 30 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- BuildRequires:
  * Add update-bootloader to resolve the choice with perl-bootloader
  * Reorder
- Remove the kernel spec macro file
  * Add macros in the specfile
* Mon Sep  2 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Provides certs and extra uki dir
- Summary fix
- Use static initrd release into version
- scriptlets:
  - use efi_dir when adding/removing uki in boot part
  - install the entry in posttrans instead of post
    Allow during the update to be install after the removal of previous entry
  - Do not fail if no entry added/removed
* Thu Aug  1 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Follow uki-tool changes
  - binary name has been changed
  - sdboot/grub2 parameters has been changed
* Wed Jul 17 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Use of needsbinariesforbuild instead of needsrootforbuild to get the static
  initrd file to build the uki  without the need of the root user.
- certification file should follow the uki name
* Thu Jul 11 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Add the kernel command line
- update condition in scriptel to guess the bootloader
- Use multibuild and build for different kernel flavors
* Mon Jul  8 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Add extra directory as specify into uapi group site
  * See: https://uapi-group.org/specifications/specs/unified_kernel_image/#locations-and-naming-for-uki-auxiliary-resources
* Mon Jul  8 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Build requires changes from ukit to uki-tool
* Wed Jul  3 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Switch from initrd-dracut-generic to static-initrd-generic
* Thu Mar  7 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- post: Print mokutil command for both bootloaders
  * Print the cert file path and name.
- swith from %%postun to %%preun
* Mon Mar  4 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Get Kernel Version from uname variable
- Use ukit create to build the UKI.
* Thu Feb 29 2024 Valentin Lefebvre <valentin.lefebvre@suse.com>
- post: add entry to sdbooot bootloader.
* Fri Dec 29 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Rename the installed binary by "uki" without kernel version
  Binary will be installed in the corresponding kernel version tree
* Mon Dec 11 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Add a requires to the kernel used to build the UKI
- Update the initrd version
* Fri Dec  1 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Update the version format
* Fri Nov 24 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Get the version of the pre-install static initrd
- Change version format to %%{kernel_version}_%%{initrd_version}
* Fri Oct 27 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Remove the grub2 mkconfig already deal by ukit
* Thu Sep 28 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Add dependency for systemd-pcrphase to wotk at boot
* Wed Sep 27 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Use UKIFY to generate keys for secure boot and PCR.
  - Stop using nedssslcertforbuild
  - Remove openSUSE certificate
- Update the UKIFY build command
  - Integrate all pcr keys in the UKI sections
  - Sign it
* Mon Sep 25 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- specfile %%postun: remove correctly grub2 menu entry
* Fri Sep 22 2023 Valentin Lefebvre <valentin.lefebvre@suse.com>
- Create the generic signed unified kernel image
